@@ -45,7 +45,7 @@ use bundle::DataItem;
 use error::Error;
 use merkle::{generate_data_root, generate_leaves, resolve_proofs};
 use status::{BundleStatus, Status, StatusCode};
-use transaction::{Base64, FromUtf8Strs, Tag, ToItems, Transaction};
+use transaction::{Base64, Chunk, FromUtf8Strs, Tag, ToItems, Transaction};
 
 const VERSION: &'static str = env!("CARGO_PKG_VERSION");
 
@@ -63,7 +63,7 @@ struct OraclePrice {
 struct OraclePricePair {
     pub usd: f32,
 }
-
+#[derive(Clone, Debug)]
 pub struct PathsChunk(Vec<PathBuf>, u64);
 
 pub fn upload_bundles_stream<'a>(
@@ -378,6 +378,40 @@ impl Arweave {
         Ok((signed_transaction.id.clone(), signed_transaction.reward))
     }
 
+    pub async fn post_transaction_chunks(
+        &self,
+        signed_transaction: Transaction,
+    ) -> Result<(Base64, u64), Error> {
+        if signed_transaction.id.0.is_empty() {
+            return Err(error::Error::UnsignedTransaction.into());
+        }
+
+        let transaction_with_no_data = signed_transaction.clone_with_no_data()?;
+        let (id, reward) = self.post_transaction(&transaction_with_no_data).await?;
+
+        let _ = try_join_all((0..signed_transaction.chunks.len()).map(|i| {
+            let chunk = signed_transaction.get_chunk(i).unwrap();
+            self.post_chunk(chunk)
+        }))
+        .await?;
+
+        Ok((id, reward))
+    }
+
+    pub async fn post_chunk(&self, chunk: Chunk) -> Result<(), Error> {
+        let url = self.base_url.join("chunk/")?;
+        let client = reqwest::Client::new();
+        let resp = client
+            .post(url)
+            .json(&chunk)
+            .header(&ACCEPT, "application/json")
+            .header(&CONTENT_TYPE, "application/json")
+            .send()
+            .await?;
+        assert_eq!(resp.status(), reqwest::StatusCode::OK);
+        Ok(())
+    }
+
     pub async fn get_pending_count(&self) -> Result<usize, Error> {
         let url = self.base_url.join("tx/pending")?;
         let tx_ids: Vec<String> = reqwest::get(url).await?.json().await?;
@@ -1074,6 +1108,7 @@ impl Arweave {
         let signed_transaction = self.sign_transaction(transaction)?;
 
         let (id, reward) = self.post_transaction(&signed_transaction).await?;
+        // let (id, reward) = self.post_transaction_chunks(signed_transaction).await?;
 
         let status = BundleStatus {
             id,
@@ -1136,24 +1171,25 @@ impl Arweave {
         transaction_id: String,
         log_dir: PathBuf,
     ) -> Result<(), Error> {
-        let mut relative_paths = Vec::<String>::new();
-        let mut id_paths = Vec::<String>::new();
+        let mut consolidated_paths = serde_json::Map::new();
         for (file_path, id_obj) in manifest["paths"].as_object().unwrap() {
-            relative_paths.push(format!(
-                "https://arweave.net/{}/{}",
-                transaction_id, file_path
-            ));
-            id_paths.push(format!(
-                "https://arweave.net/{}",
-                id_obj["id"].as_str().unwrap()
-            ));
+            let id = id_obj["id"].as_str().unwrap();
+            consolidated_paths.insert(
+                file_path.to_owned(),
+                json!({"id": id, "relative_url": format!(
+                    "https://arweave.net/{}/{}",
+                    transaction_id, file_path
+                ), "id_url": format!(
+                    "https://arweave.net/{}",
+                    id
+                )}),
+            );
         }
-        let value = json!({"relative_paths": relative_paths, "id_paths": id_paths});
         fs::write(
             log_dir
                 .join(format!("manifest_{}", transaction_id))
                 .with_extension("json"),
-            serde_json::to_string(&value)?,
+            serde_json::to_string(&json!(consolidated_paths))?,
         )
         .await?;
         Ok(())
